from pathlib import Path
from typing import Any

import numpy as np


def compare_assets(exp: dict[str, Any], ref: dict[str, Any]) -> None:
    assert exp.keys() == ref.keys()
    for k in exp.keys():
        if k != "href":
            assert exp[k] == ref[k]
        else:
            exp_p = Path(exp[k])
            ref_p = Path(ref[k])
            assert exp_p == ref_p


def compare_items(exp: dict[str, Any], ref: dict[str, Any]) -> None:
    assert exp["id"] == ref["id"]
    np.testing.assert_array_almost_equal(exp["bbox"], ref["bbox"])
    assert exp["geometry"].keys() == ref["geometry"].keys()
    assert exp["geometry"]["type"] == ref["geometry"]["type"]
    np.testing.assert_array_almost_equal(
        exp["geometry"]["coordinates"], ref["geometry"]["coordinates"]
    )
    assert exp["properties"] == ref["properties"]
    assert exp["assets"].keys() == ref["assets"].keys()
    for k in exp["assets"].keys():
        compare_assets(exp["assets"][k], ref["assets"][k])


def compare_extent(exp: dict[str, Any], ref: dict[str, Any]) -> None:
    assert exp["extent"]["temporal"] == ref["extent"]["temporal"]
    np.testing.assert_array_almost_equal(
        exp["extent"]["spatial"]["bbox"], ref["extent"]["spatial"]["bbox"]
    )



import datetime
import json
from pathlib import Path

import pytest

from stac_generator.core.base.generator import CollectionGenerator
from stac_generator.core.base.schema import StacCollectionConfig
from stac_generator.core.base.utils import read_source_config
from stac_generator.core.raster.generator import RasterGenerator
from stac_generator.core.raster.schema import BandInfo, RasterConfig
from stac_generator.exceptions import SourceAssetException
from tests.utils import compare_extent, compare_items

CONFIG_JSON = Path("tests/files/integration_tests/raster/config/raster_config.json")


GENERATED_DIR = Path("tests/files/integration_tests/raster/generated")


JSON_CONFIGS = read_source_config(str(CONFIG_JSON))
ITEM_IDS = [item["id"] for item in JSON_CONFIGS]


@pytest.fixture(scope="module")
def raster_generators() -> list[RasterGenerator]:
    return [RasterGenerator(config) for config in JSON_CONFIGS]


@pytest.fixture(scope="module")
def collection_generator(raster_generators: list[RasterGenerator]) -> CollectionGenerator:
    return CollectionGenerator(StacCollectionConfig(id="collection"), generators=raster_generators)


@pytest.mark.parametrize("item_idx", range(len(JSON_CONFIGS)), ids=ITEM_IDS)
def test_generator_given_item_expects_matched_generated_item(
    item_idx: int, raster_generators: list[RasterGenerator]
) -> None:
    config = JSON_CONFIGS[item_idx]
    expected_path = GENERATED_DIR / f"{config['id']}/{config['id']}.json"
    with expected_path.open() as file:
        expected = json.load(file)
    actual = raster_generators[item_idx].generate().to_dict()
    compare_items(expected, actual)


def test_collection_generator(collection_generator: CollectionGenerator) -> None:
    actual = collection_generator().to_dict()
    expected_path = GENERATED_DIR / "collection.json"
    with expected_path.open() as file:
        expected = json.load(file)
    compare_extent(expected, actual)


def test_given_non_existent_location_expects_raise_SourceAssetException() -> None:
    ts = datetime.datetime.now()
    config = RasterConfig(
        id="item",
        location="non_existent.tif",
        collection_date=ts.date(),
        collection_time=ts.time(),
        band_info=[BandInfo(name="Invalid_Band")],
    )
    item_generator = RasterGenerator(config)
    with pytest.raises(SourceAssetException):
        item_generator.generate()


def test_given_no_band_info_expects_raises() -> None:
    with pytest.raises(ValueError):
        RasterConfig.model_validate(
            {
                "id": "item",
                "location": "non_existent.tif",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
            }
        )


def test_given_invalid_common_name_expects_raises() -> None:
    with pytest.raises(ValueError):
        RasterConfig.model_validate(
            {
                "id": "item",
                "location": "non_existent.tif",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "band_info": [{"name": "band1", "common_name": "band1"}],
            }
        )


def test_given_valid_common_name_capital_expects_lowered() -> None:
    config = RasterConfig.model_validate(
        {
            "id": "item",
            "location": "non_existent.tif",
            "collection_date": "2020-01-01",
            "collection_time": "00:00",
            "band_info": [{"name": "RED", "common_name": "RED"}],
        }
    )
    assert config.band_info[0].name == "red"
    assert config.band_info[0].common_name == "red"


def test_given_valid_pathlibPath_expects_correct_serialisation() -> None:
    ts = datetime.datetime.now()
    config = RasterConfig(
        id="time",
        location=Path("/path/to/file.tif"),  # type: ignore[arg-type]
        collection_date=ts.date(),
        collection_time=ts.time(),
        band_info=[BandInfo(name="Invalid_Band")],
    )
    assert config.location == "/path/to/file.tif"


def test_given_invalid_path_type_expects_raises() -> None:
    with pytest.raises(ValueError):
        ts = datetime.datetime.now()
        RasterConfig(
            id="time",
            location=123,  # type: ignore[arg-type]
            collection_date=ts.date(),
            collection_time=ts.time(),
            band_info=[BandInfo(name="Invalid_Band")],
        )



r"""
  /$$$$$$ /$$$$$$$$/$$$$$$  /$$$$$$         /$$$$$$ /$$$$$$$$/$$   /$$
 /$$__  $|__  $$__/$$__  $$/$$__  $$       /$$__  $| $$_____| $$$ | $$
| $$  \__/  | $$ | $$  \ $| $$  \__/      | $$  \__| $$     | $$$$| $$
|  $$$$$$   | $$ | $$$$$$$| $$            | $$ /$$$| $$$$$  | $$ $$ $$
 \____  $$  | $$ | $$__  $| $$            | $$|_  $| $$__/  | $$  $$$$
 /$$  \ $$  | $$ | $$  | $| $$    $$      | $$  \ $| $$     | $$\  $$$
|  $$$$$$/  | $$ | $$  | $|  $$$$$$/      |  $$$$$$| $$$$$$$| $$ \  $$
 \______/   |__/ |__/  |__/\______/        \______/|________|__/  \__/
"""  # noqa: D212

from __future__ import annotations

import logging
from argparse import ArgumentParser, Namespace, _SubParsersAction

from pydantic import ValidationError
from rich_argparse import RawDescriptionRichHelpFormatter

from stac_generator.__version__ import __version__

root_logger = logging.getLogger("stac_generator")
logger = logging.getLogger(__name__)


def log_exception(e: Exception, show_stack_trace: bool = False) -> None:
    logger.exception(e, exc_info=show_stack_trace)
    if not show_stack_trace:
        logger.info("Run the command with -v to show detailed error.")


def serialise_handler(args: Namespace) -> None:
    from stac_generator.cli.serialise import serialise_handler as _serialise_handler

    show_stack_trace = False
    if args.v:
        root_logger.setLevel(logging.DEBUG)
        show_stack_trace = True

    # CLI args take precedence over metadata fields
    try:
        _serialise_handler(
            id=args.id,
            src=args.src,
            dst=args.dst,
            title=args.title,
            description=args.description,
            license=args.license,
            platform=args.platform,
            instruments=args.instruments,
            constellation=args.constellation,
            mission=args.mission,
            num_workers=args.num_workers,
        )
    except ValidationError as e:
        logger.info(
            "Error encountered while parsing config. Fix the error by addressing the following:"
        )
        log_exception(e, show_stack_trace)
    except Exception as e:  # noqa: BLE001
        log_exception(e, show_stack_trace)


def add_serialise_sub_command(sub_parser: _SubParsersAction) -> None:
    parser = sub_parser.add_parser("serialise", help="Generate STAC record")
    # Source commands
    parser.add_argument(
        "src",
        type=str,
        action="extend",
        nargs="+",
        help="""path to the source_config.
                Path can be a local path or a url.
                Path also accepts multiple values.
                Source config contains metadata specifying how a raw file is read.
                At the minimum, it must contain the file location.
                To learn more about source config, please visit INSERT_DOC_URL.
            """,
    )
    parser.add_argument(
        "--dst",
        type=str,
        default="generated",
        help="""path to where the generated collection is stored.
                Accepts a local path or a remote api endpoint.
                If path is local, collection and item json files will be written to disk.
                If path is an endpoint, the collection and item json files will be pushed using STAC api methods.
                If not value is provided, the folder generated will be created in the current path to store generated records
            """,
    )

    parser.add_argument("-v", action="store_true", help="increase verbosity for debugging")
    # Collection Information
    collection_metadata = parser.add_argument_group("STAC collection metadata")
    collection_metadata.add_argument(
        "--id", type=str, help="id of collection", default="Collection"
    )
    collection_metadata.add_argument(
        "--title", type=str, help="title of collection", required=False, default="Auto-generated."
    )
    collection_metadata.add_argument(
        "--description",
        type=str,
        help="description of collection",
        required=False,
        default="Auto-generated",
    )

    # STAC Common Metadata
    common_metadata = parser.add_argument_group("STAC common metadata")
    common_metadata.add_argument(
        "--license", type=str, help="STAC license", required=False, default="proprietary"
    )
    common_metadata.add_argument(
        "--platform", type=str, help="STAC platform", required=False, default=None
    )
    common_metadata.add_argument(
        "--constellation", type=str, help="STAC constellation", required=False, default=None
    )
    common_metadata.add_argument(
        "--mission", type=str, help="STAC mission", required=False, default=None
    )
    common_metadata.add_argument(
        "--instruments",
        action="extend",
        nargs="+",
        type=str,
        required=False,
        default=None,
        help="STAC instrument",
    )

    # Serialiser metadata
    serialiser_metadata = parser.add_argument_group("Serialliser metadata")
    serialiser_metadata.add_argument(
        "--num_workers",
        type=int,
        required=False,
        default=1,
        help="Number of threads to use for serialisation. If 1, serialisation will be done in a single thread.",
    )
    parser.set_defaults(func=serialise_handler)


def run_cli() -> None:
    # Build the CLI argument parser
    parser = ArgumentParser(
        prog="stac_generator",
        description=__doc__,
        formatter_class=RawDescriptionRichHelpFormatter,
    )
    parser.add_argument("-V", "--version", action="version", version=__version__)
    sub_parser = parser.add_subparsers(dest="command", help="Sub commands")
    add_serialise_sub_command(sub_parser)
    args = parser.parse_args()

    if args.command == "serialise":
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    run_cli()



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Sequence

    from stac_pydantic.shared import Provider


def serialise_handler(
    id: str,
    src: str | Sequence[str],
    dst: str,
    title: str | None = "Auto-generated Stac Item",
    description: str | None = "Auto-generated Stac Item",
    license: str | None = None,
    platform: str | None = None,
    instruments: list[str] | None = None,
    constellation: str | None = None,
    mission: str | None = None,
    providers: list[Provider] | None = None,
    num_workers: int = 1,
) -> None:
    from concurrent.futures import ProcessPoolExecutor

    from stac_generator.core.base.generator import StacSerialiser
    from stac_generator.core.base.schema import StacCollectionConfig
    from stac_generator.factory import StacGeneratorFactory

    collection_config = StacCollectionConfig(
        id=id,
        title=title,
        description=description,
        license=license,
        platform=platform,
        constellation=constellation,
        mission=mission,
        instruments=instruments,
        providers=providers,
    )

    # Generate
    if num_workers == 1:
        # Use a single thread
        generator = StacGeneratorFactory.get_collection_generator(
            source_configs=src,
            collection_config=collection_config,
        )
        # Save
        serialiser = StacSerialiser(generator, dst)
        serialiser()
    elif num_workers > 1:
        with ProcessPoolExecutor(max_workers=num_workers) as executor:
            generator = StacGeneratorFactory.get_collection_generator(
                source_configs=src,
                collection_config=collection_config,
                pool=executor,
            )
            serialiser = StacSerialiser(generator, dst)
            serialiser()
    else:
        raise ValueError(f"Invalid number of threads: {num_workers}. Must be greater than 0.")



import json
from pathlib import Path

import pytest
from pystac import Collection

from stac_generator.cli.serialise import serialise_handler
from tests.utils import compare_extent, compare_items


@pytest.mark.parametrize(
    "id,title,description,license,platform,instruments,constellation,mission,num_workers",
    [
        ("my_collection", "my_title", "my_description", "MIT", None, None, None, None, 1),
        ("my_collection", None, "my_description", "MIT", None, None, None, None, 4),
    ],
)
def test_serialise(
    id: str,
    title: str | None,
    description: str | None,
    license: str | None,
    platform: str | None,
    instruments: list[str],
    constellation: str | None,
    mission: str | None,
    num_workers: int,
    tmp_path: Path,
) -> None:
    dst = tmp_path / "generated"
    serialise_handler(
        id=id,
        src="tests/files/integration_tests/composite/config/composite_config.json",
        dst=dst.as_posix(),
        title=title,
        description=description,
        license=license,
        platform=platform,
        instruments=instruments,
        constellation=constellation,
        mission=mission,
        num_workers=num_workers,
    )
    generated_path = Path("tests/files/integration_tests/composite/generated")
    expected_collection_path = generated_path / "collection.json"
    with expected_collection_path.open() as file:
        expected_collection = json.load(file)

    actual_collection_path = dst / "collection.json"
    with actual_collection_path.open() as file:
        actual_collection = json.load(file)
    compare_extent(expected_collection, actual_collection)
    collection = Collection.from_file(actual_collection_path)
    assert collection.id == id
    assert collection.title == title
    assert collection.description == description
    assert collection.license == license
    for item in collection.get_items(recursive=True):
        config_loc = dst / f"{item.id}/{item.id}.json"
        with config_loc.open("r") as file:
            expected = json.load(file)
        actual = item.to_dict()
        compare_items(expected, actual)


@pytest.mark.parametrize("num_workers", [0, -1, -2])
def test_given_invalid_num_workers_expect_raises(num_workers: int) -> None:
    with pytest.raises(ValueError):
        serialise_handler(
            id="Collection",
            src="tests/files/integration_tests/composite/config/composite_config.json",
            dst="generated",
            title=None,
            description=None,
            license=None,
            platform=None,
            instruments=None,
            constellation=None,
            mission=None,
            num_workers=num_workers,
        )



import datetime
import json
from pathlib import Path

import pytest

from stac_generator.core.base.generator import CollectionGenerator
from stac_generator.core.base.schema import StacCollectionConfig
from stac_generator.core.base.utils import read_source_config
from stac_generator.core.vector.generator import VectorGenerator
from stac_generator.core.vector.schema import JoinConfig, VectorConfig
from stac_generator.exceptions import SourceAssetException, StacConfigException
from tests.utils import compare_extent, compare_items

CONFIG_JSON = Path("tests/files/integration_tests/vector/config/vector_config.json")


GENERATED_DIR = Path("tests/files/integration_tests/vector/generated")


CONFIGS = read_source_config(str(CONFIG_JSON))
ITEM_IDS = [item["id"] for item in CONFIGS]


@pytest.fixture(scope="module")
def vector_generators() -> list[VectorGenerator]:
    return [VectorGenerator(config) for config in CONFIGS]


@pytest.fixture(scope="module")
def collection_generator(vector_generators: list[VectorGenerator]) -> CollectionGenerator:
    return CollectionGenerator(StacCollectionConfig(id="collection"), generators=vector_generators)


@pytest.mark.parametrize("item_idx", range(len(CONFIGS)), ids=ITEM_IDS)
def test_generator_given_item_expects_matched_generated_item(
    item_idx: int, vector_generators: list[VectorGenerator]
) -> None:
    config = CONFIGS[item_idx]
    expected_path = GENERATED_DIR / f"{config['id']}/{config['id']}.json"
    with expected_path.open() as file:
        expected = json.load(file)
    actual = vector_generators[item_idx].generate().to_dict()
    compare_items(expected, actual)


def test_collection_generator(collection_generator: CollectionGenerator) -> None:
    actual = collection_generator().to_dict()
    expected_path = GENERATED_DIR / "collection.json"
    with expected_path.open() as file:
        expected = json.load(file)
    compare_extent(expected, actual)


def test_given_non_existent_join_location_expects_raise_SourceAssetException() -> None:
    config = {
        "id": "valid_vector",
        "location": "tests/files/unit_tests/vectors/Werribee.geojson",
        "collection_date": "2025-01-01",
        "collection_time": "00:00:00",
        "column_info": [{"name": "Suburb_Name", "description": "Suburb_Name"}],
        "join_config": {
            "file": "Non-Existent.csv",
            "left_on": "Suburb_Name",
            "right_on": "Area",
            "column_info": [
                {"name": "Area", "description": "Area name"},
                {"name": "Distance", "description": "Driving Distance to CBD in km"},
                {
                    "name": "Public_Transport",
                    "description": "Time taken to reach CBD by public transport in minutes",
                },
                {"name": "Drive", "description": "Time taken to reach CBD by driving in minutes"},
                {"name": "Growth", "description": "Average 5 year growth in percentage in 2025"},
                {"name": "Yield", "description": "Average rental yield in 2025"},
            ],
        },
    }
    item_generator = VectorGenerator(config)
    with pytest.raises(SourceAssetException):
        item_generator.generate()


def test_given_non_existent_location_expects_raise_SourceAssetException() -> None:
    ts = datetime.datetime.now()
    config = VectorConfig(
        id="item",
        location="non_existent.shp",
        collection_date=ts.date(),
        collection_time=ts.time(),
    )
    item_generator = VectorGenerator(config)
    with pytest.raises(SourceAssetException):
        item_generator.generate()


def test_given_empty_join_column_info_expects_raises() -> None:
    with pytest.raises(ValueError):
        JoinConfig.model_validate(
            {
                "file": "non_existent.csv",
                "left_on": "column",
                "right_on": "column",
                "column_info": [],
            }
        )


def test_given_duplicated_id_expects_raises() -> None:
    config = [
        {
            "id": "valid_vector",
            "location": "tests/files/unit_tests/vectors/Werribee.geojson",
            "collection_date": "2025-01-01",
            "collection_time": "00:00:00",
            "column_info": [{"name": "Suburb_Name", "description": "Suburb_Name"}],
        },
        {
            "id": "valid_vector",
            "location": "tests/files/unit_tests/vectors/Werribee.geojson",
            "collection_date": "2025-01-01",
            "collection_time": "00:00:00",
            "column_info": [{"name": "Suburb_Name", "description": "Suburb_Name"}],
        },
    ]
    with pytest.raises(StacConfigException):
        generators = [VectorGenerator(cfg) for cfg in config]
        CollectionGenerator(StacCollectionConfig(id="Collecton"), generators)



import datetime
import json
from collections.abc import Generator
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path
from typing import Any

import pytest

from stac_generator.core.base import StacCollectionConfig
from stac_generator.core.base.generator import CollectionGenerator
from stac_generator.core.base.schema import SourceConfig
from stac_generator.core.point.schema import PointConfig
from stac_generator.core.vector import VectorGenerator
from stac_generator.core.vector.schema import VectorConfig
from stac_generator.factory import StacGeneratorFactory
from tests.utils import compare_extent, compare_items

FILE_PATH = Path("tests/files/integration_tests")
GENERATED_PATH = FILE_PATH / "composite/generated"
COLLECTION_ID = "collection"
collection_config = StacCollectionConfig(id=COLLECTION_ID)
CONFIGS_LIST = [
    str(FILE_PATH / "point/config/point_config.json"),
    str(FILE_PATH / "vector/config/vector_config.json"),
    str(FILE_PATH / "raster/config/raster_config.json"),
]
COMPOSITE_CONFIG = FILE_PATH / "composite/config/composite_config.json"


class DbfConfig(SourceConfig): ...


@pytest.fixture(scope="module")
def dbf_config() -> DbfConfig:
    return DbfConfig.model_validate(
        {
            "id": "item",
            "collection_date": "2020-01-01",
            "collection_time": "00:00",
            "location": "data.dbf",
        }
    )


@pytest.fixture(scope="module")
def composite_generator() -> CollectionGenerator:
    return StacGeneratorFactory.get_collection_generator(str(COMPOSITE_CONFIG), collection_config)


@pytest.fixture(scope="module")
def list_generator() -> CollectionGenerator:
    return StacGeneratorFactory.get_collection_generator(CONFIGS_LIST, collection_config)


@pytest.fixture(scope="module")
def threadpool_generator() -> Generator[CollectionGenerator, None, None]:
    executor = ThreadPoolExecutor(max_workers=4)
    # Use the executor to create a thread pool for the generator
    yield StacGeneratorFactory.get_collection_generator(
        CONFIGS_LIST,
        collection_config,
        pool=executor,
    )
    # Cleanup
    executor.shutdown(wait=True)


@pytest.mark.parametrize(
    "generator_fx",
    (
        "composite_generator",
        "list_generator",
        "threadpool_generator",
    ),
    ids=[
        "Composite Config",
        "List Configs",
        "ThreadPool Config",
    ],
)
def test_generator_factory(
    generator_fx: str,
    request: pytest.FixtureRequest,
) -> None:
    generator: CollectionGenerator = request.getfixturevalue(generator_fx)
    collection = generator()
    expected_collection_path = GENERATED_PATH / "collection.json"
    with expected_collection_path.open() as file:
        expected_collection = json.load(file)
    actual_collection = collection.to_dict()
    compare_extent(expected_collection, actual_collection)
    for item in collection.get_items(recursive=True):
        config_loc = GENERATED_PATH / f"{item.id}/{item.id}.json"
        with config_loc.open("r") as file:
            expected = json.load(file)
        actual = item.to_dict()
        compare_items(expected, actual)


@pytest.mark.parametrize(
    "config",
    [(123), (45.6), ([23, 4.5]), (None)],
)
def test_given_invalid_config_type_expects_raises(config: Any) -> None:
    with pytest.raises(TypeError):
        StacGeneratorFactory.get_item_generators(config)


@pytest.mark.parametrize(
    "config",
    [
        ({"id": "Missing Location"}),  # Missing Location
        ({"location": "data.csv"}),  # Missing ID
        (
            [{"id": "Full Item", "location": "data.geotiff"}, {"id": "Missing Location"}]
        ),  # Missing Location - list
        (
            [{"id": "Full Item", "location": "data.geotiff"}, {"location": "MissingID.csv"}]
        ),  # Missing ID - list
    ],
)
def test_given_dict_missing_id_location_expects_raises(
    config: dict[str, Any] | list[dict[str, Any]],
) -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.get_item_generators(config)


@pytest.mark.parametrize(
    "config",
    [
        # Raster config
        (
            {
                "id": "raster",
                "location": "data.geotiff",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
            },  # Raster - Missing BandInfo
        ),
        (
            {
                "id": "raster",
                "location": "data.geotiff",
                "collection_date": "2020-01-01",
                "band_info": [{"name": "red", "common_name": "red"}],
            },  # Raster - Missing collection time
        ),
        (
            {
                "id": "raster",
                "location": "data.geotiff",
                "collection_time": "00:00",
                "band_info": [{"name": "red", "common_name": "red"}],
            },  # Raster - Missing collection time
        ),
        (
            {
                "id": "raster",
                "location": "data.geotiff",
                "collection_date": "2025-01-05",
                "collection_time": "00:00",
                "band_info": [{"name": "red", "common_name": "red_ms"}],
            },  # Raster - invalid common name
        ),
        # Point config
        (
            {
                "id": "point",
                "location": "data.csv",
                "collection_time": "00:00",
                "X": "lat",
                "Y": "lon",
            },  # Point - missing date
        ),
        (
            {
                "id": "point",
                "location": "data.csv",
                "collection_date": "2020-01-01",
                "X": "lat",
                "Y": "lon",
            },  # Point - missing time
        ),
        (
            {
                "id": "point",
                "location": "data.csv",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "Y": "lon",
            },  # Point - missing X
        ),
        (
            {
                "id": "point",
                "location": "data.csv",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "X": "lat",
            },  # Point - missing Y
        ),
        # Vector config
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_time": "00:00",
            },  # vector missing date
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
            },  # vector missing time
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "column_info": [{"name": "name"}, {"name": "code"}],
                "join_config": {
                    "left_on": "name",
                    "right_on": "name",
                    "column_info": [{"name": "name"}, {"name": "area"}],
                },
            },  # vector join config - join config - missing file
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "column_info": [{"name": "name"}, {"name": "code"}],
                "join_config": {
                    "file": "join_file.csv",
                    "right_on": "name",
                    "column_info": [{"name": "name"}, {"name": "area"}],
                },
            },  # vector join config - join config - missing left on
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "column_info": [{"name": "name"}, {"name": "code"}],
                "join_config": {
                    "file": "join_file.csv",
                    "left_on": "name",
                    "column_info": [{"name": "name"}, {"name": "area"}],
                },
            },  # vector join config - join config - missing right on
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "column_info": [{"name": "name"}, {"name": "code"}],
                "join_config": {
                    "file": "join_file.csv",
                    "left_on": "name",
                    "right_on": "name",
                },
            },  # vector join config - join config - missing column info
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "column_info": [{"name": "code"}],
                "join_config": {
                    "file": "join_file.csv",
                    "left_on": "name",
                    "right_on": "name",
                    "column_info": [{"name": "name"}, {"name": "area"}],
                },
            },  # vector join config - join config - left on not in column info
        ),
        (
            {
                "id": "vector",
                "location": "data.geojson",
                "collection_date": "2020-01-01",
                "collection_time": "00:00",
                "column_info": [{"name": "name"}, {"name": "code"}],
                "join_config": {
                    "file": "join_file.csv",
                    "left_on": "name",
                    "right_on": "name",
                    "column_info": [{"name": "area"}],
                },
            },  # vector join config - join config - right on not in join column info
        ),
    ],
)
def test_given_dict_missing_important_fields_expects_raises(
    config: dict[str, Any] | list[dict[str, Any]],
) -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.get_item_generators(config)


def test_given_unregistered_config_ext_expects_raises() -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.get_extension_config_handler("dbf")


def test_given_unregistered_ext_expects_raises() -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.get_extension_handler("dbf")


def test_given_unregistered_config_class_expects_raises(dbf_config: DbfConfig) -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.get_generator_handler(dbf_config)


def test_given_registered_config_expects_no_error() -> None:
    StacGeneratorFactory.register_extension_handler("dbf", DbfConfig)
    assert StacGeneratorFactory.get_extension_handler("dbf") == DbfConfig


def test_given_registered_generator_expects_no_error(dbf_config: DbfConfig) -> None:
    StacGeneratorFactory.register_generator_handler(DbfConfig, VectorGenerator)
    assert StacGeneratorFactory.get_generator_handler(dbf_config) == VectorGenerator


def test_register_existing_ext_no_force_exp_raises() -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.register_extension_handler("csv", DbfConfig)


def test_register_invalid_handler_exp_raises() -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.register_extension_handler("dbf", VectorGenerator, force=True)  # type: ignore[arg-type]


def test_register_existing_ext_with_force_no_raises() -> None:
    StacGeneratorFactory.register_extension_handler("csv", DbfConfig, force=True)
    assert StacGeneratorFactory.get_extension_handler("csv") == DbfConfig


def test_register_existing_config_no_force_exp_raises() -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.register_generator_handler(PointConfig, VectorGenerator)


def test_register_existing_config_force_no_raises() -> None:
    StacGeneratorFactory.register_generator_handler(VectorConfig, VectorGenerator, force=True)
    assert (
        StacGeneratorFactory.get_generator_handler(
            VectorConfig(
                id="vector",
                collection_date=datetime.date(2020, 1, 1),
                collection_time=datetime.time(0, 0),
                location="data.geojson",
            )
        )
        == VectorGenerator
    )


def test_register_invalid_config_handler_exp_raises() -> None:
    with pytest.raises(ValueError):
        StacGeneratorFactory.register_generator_handler(VectorConfig, VectorConfig, force=True)  # type: ignore[arg-type]



from __future__ import annotations

from typing import Annotated, Any, Literal

from pydantic import AfterValidator, BaseModel, BeforeValidator

from stac_generator.core.base.schema import SourceConfig

VALID_COMMON_NAME = Literal[
    "coastal",
    "blue",
    "green",
    "red",
    "rededge",
    "yellow",
    "pan",
    "nir",
    "nir08",
    "nir09",
    "cirrus",
    "swir16",
    "swir22",
    "lwir",
    "lwir11",
    "lwir12",
]


class BandInfo(BaseModel):
    """Band information for raster data"""

    name: Annotated[str, AfterValidator(lambda name: name.lower())]
    common_name: Annotated[
        VALID_COMMON_NAME | None,
        BeforeValidator(
            lambda common_name: common_name.lower() if isinstance(common_name, str) else None
        ),
    ] = None
    wavelength: float | None = None
    nodata: float | None = None
    data_type: str | None = None
    description: str | None = None


class RasterOwnConfig(BaseModel):
    """Configuration for raster data sources"""

    band_info: list[BandInfo]
    """List of band information - REQUIRED"""


class RasterConfig(SourceConfig, RasterOwnConfig):
    """Configuration for raster data sources"""

    def to_asset_config(self) -> dict[str, Any]:
        return RasterOwnConfig.model_construct(
            **self.model_dump(mode="json", exclude_none=True, exclude_unset=True)
        ).model_dump(mode="json", exclude_none=True, exclude_unset=True, warnings=False)



from stac_generator.core.vector.generator import VectorGenerator
from stac_generator.core.vector.schema import VectorConfig, VectorOwnConfig

__all__ = ("VectorConfig", "VectorGenerator", "VectorOwnConfig")



from stac_generator.core.raster.generator import RasterGenerator
from stac_generator.core.raster.schema import RasterConfig, RasterOwnConfig

__all__ = ("RasterConfig", "RasterGenerator", "RasterOwnConfig")



import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setLevel(logging.DEBUG)
formatter = logging.Formatter(
    fmt="%(asctime)s-%(name)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%dT%H:%M:%S",
)
handler.setFormatter(formatter)
logger.addHandler(handler)
